/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BarracudaAdmin.java,v 1.6 2002/06/21 17:57:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.admin;

import java.io.*;
import java.util.*;
import java.lang.ref.*;
import java.net.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.util.dom.io.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.event.helper.*;
import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.http.*;
import org.enhydra.barracuda.core.util.srv.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;

import org.enhydra.barracuda.admin.events.*;
import org.enhydra.barracuda.admin.xmlc.*;

/**
 * This mini-app is can be used to administer Barracuda applications
 */
public class BarracudaAdmin extends DefaultEventGateway {

    //public constants
    protected static Logger logger = Logger.getLogger(BarracudaAdmin.class.getName());

    public static String DEFAULT_ADMIN_PROPS = "org.enhydra.barracuda.admin.AdminDefs";

    private static final String CMD_STR = "$cmd";
    private static final String CMD_RESULT = "$cmd_result";
    private static String sep = System.getProperty("line.separator"); 

    protected ResourceBundle rb = null;
    protected Class templateClass = AdminHTML.class;
    protected Map cmdMap = new HashMap();

    //this defines the various event handlers
    private ListenerFactory getAdminFactory =            new DefaultListenerFactory() {public BaseEventListener getInstance() {return new GetAdminHandler();}            public String getListenerID() {return getID(GetAdminHandler.class);}};
    private ListenerFactory renderAdminFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderAdminHandler();}        public String getListenerID() {return getID(RenderAdminHandler.class);}};                

    //------------------- BarracudaAdmin ------------------------
    /**
     * Public constructor
     */
    public BarracudaAdmin() {
        //specify who's interested in what
        specifyLocalEventInterests(getAdminFactory, GetAdmin.class);
        specifyLocalEventInterests(renderAdminFactory, RenderAdmin.class);
        
        //setup the default admin file
        setAdminFile(DEFAULT_ADMIN_PROPS);
    }

    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * GetAdminHandler - handle the request to get the config 
     * screen.
     */
    class GetAdminHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //check to see if there are any commands associated with the request
            String cmdParm = context.getRequest().getParameter(CMD_STR);
            Object cmdResult = null;
            if (cmdParm!=null) {
                Object o = cmdMap.get(cmdParm);
                if (o!=null) {
                    //now that we know we have a valid command, we need to 
                    //handle it according to its type...
                    
                    //...if its an actual command
                    if (o instanceof TargetCommand) {
                        TargetCommand command = (TargetCommand) o;
                        cmdResult = command.handleCommand();
                    
                    //...otherwise it must be a URL
                    } else if (o instanceof TargetURL) {
                        TargetURL turl = (TargetURL) o;
                        cmdResult = turl.handleURL();
                    }
                }
            }

            //forward control to the renderer
            context.putState(CMD_RESULT, cmdResult);
            context.getQueue().addEvent(new RenderAdmin());
        }
    }
    


    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderAdminHandler - this is where we handle any RenderAdmin 
     * event and actually generate the view. 
     */
    class RenderAdminHandler extends BTemplateViewHandler {
        public Object getTemplateModels() {return new LocalTemplateModel();}
        public Class getTemplateClass() {return templateClass;}
    }
    
    
    //------------------------------------------------------------
    //                Components - TemplateModel
    //------------------------------------------------------------
    /**
     * LocalTemplateModel
     */
    class LocalTemplateModel extends AbstractTemplateModel {
        int cntr = -1;
        int max = 5;
    
        //register the model by name
        public String getName() {return "Admin";}

        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            logger.debug("getItem, key: "+key);
            
            //start by looking for the key in the resource bundle
            if (rb!=null) {
                //look for the url key in the resource bundle;
                String url = null;
                try {url = rb.getString(key+".url");}
                catch (MissingResourceException e) {}
                
                //if we find a url param, assume we are dealing with a url;
                //store the url and return a link with the key as a
                //cmd parameter                
                if (url!=null) {
                    logger.debug("found key: "+url);
                    TargetURL turl = new TargetURL();
                    turl.url = url;
                    try {turl.params = rb.getString(key+".params");} 
                    catch (MissingResourceException e) {}
                    try {turl.method = rb.getString(key+".method");} 
                    catch (MissingResourceException e) {
                        try {turl.method = rb.getString("Default.method");} 
                        catch (MissingResourceException e2) {}
                    }
                    try {turl.user = rb.getString(key+".user");} 
                    catch (MissingResourceException e) {
                        try {turl.user = rb.getString("Default.user");} 
                        catch (MissingResourceException e2) {}
                    }
                    try {turl.pwd = rb.getString(key+".pwd");} 
                    catch (MissingResourceException e) {
                        try {turl.pwd = rb.getString("Default.pwd");} 
                        catch (MissingResourceException e2) {}
                    }
//System.out.println("URL:"+turl);                    
                    cmdMap.put(key, turl);
                    BLink blink = new BLink(null, new GetAdmin());
                    blink.setParam(CMD_STR, key);
                    return blink;
                    
                //if not found, look for cmd
                } else {
                    TargetCommand command = new TargetCommand();
                    try {command.cmd = rb.getString(key+".cmd");}
                    catch (MissingResourceException e) {}
                    if (command.cmd!=null) {
                        logger.debug("found cmd: "+command.cmd);
                        command.path = new File(rb.getString(key+".path"));
//System.out.println("Cmd:"+command);                    
                        cmdMap.put(key, command);
                        BLink blink = new BLink(null, new GetAdmin());
                        blink.setParam(CMD_STR, key);
                        return blink;
                    }
                }
            } else {
                logger.warn("resource bundle missing");
            }
            
            //if we didn't find the key in the resource bundle, handle normally            
            if (key.equals("CommandResults")) {
                EventContext ec = vc. getEventContext();
                Object cmdResult = ec.getState(CMD_RESULT);
                if (cmdResult==null) {
                    return "";
                } else if (cmdResult instanceof List) {
                    List l = (List) cmdResult;
                    StringBuffer sb = new StringBuffer(500);
                    for (int i=0; i<l.size(); i++) {
                        sb.append(l.get(i)+sep);
                    }
                    return sb.toString();
                } else if (cmdResult instanceof Exception) {
                    StringWriter sw = new StringWriter();
                    ((Exception) cmdResult).printStackTrace(new PrintWriter(sw));
                    return sw.toString();
                } else {
                    return cmdResult.toString();
                }
            } else {
                return super.getItem(key);
            }
        }
    }



    //------------------------------------------------------------
    //                Utility Stuff (overridable)
    //------------------------------------------------------------
    public void setAdminFile(String s) {
        //first look for     
        try {
            logger.info("setting admin file: "+s);
            rb = ResourceBundle.getBundle(s);
        } catch (Exception e) {
            logger.error("Unable to locate resource bundle "+s+": "+e);
        }   
    }

    public void setTemplateClass(String s) {
        try {
            logger.info("setting template class: "+s);
//jrk_20020414.1_start
            //Class.forName is java1. Since Java2, use 
            //Thread.currentThread().getContextClassLoader().loadClass("MyClass")
            //This should fix some classloading issues in Engines with multiple
            //class loaders (eg.. Tomcat-3.3.x and Tomcat-4.x.x)
            //templateClass = Class.forName(s);
            templateClass = Thread.currentThread().getContextClassLoader().loadClass(s);
//jrk_20020414.1_end
        } catch (Exception e) {
            logger.warn("Unable to locate class:"+s);        
            logger.warn("Using "+templateClass+" instead");        
        }
    }

    class TargetCommand {
        public File path = null;
        public String cmd = null;
        
        public Object handleCommand() {
            logger.info("Executing cmd: "+this);
            try {
                Process p = Runtime.getRuntime().exec(cmd, null, path);
//                Process p = Runtime.getRuntime().exec("E:/WebProjects/lutris/cvs/Barracuda/src/ant.bat admin.sample1", null, path);
                List results = new ArrayList(100);
                BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
                while (true) {
                    String s = br.readLine();
                    if (s==null) break;
//                    results.add(s);
                    while (true) {
                        if (s.length()>132) {
                            results.add(s.substring(0,132));
                            s = s.substring(133);
                        } else {
                            results.add(s);
                            break;
                        }
                    }
                }
                br.close();
                br = new BufferedReader(new InputStreamReader(p.getErrorStream()));
                while (true) {
                    String s = br.readLine();
                    if (s==null) break;
//                    results.add(s);
                    while (true) {
                        if (s.length()>132) {
                            results.add(s.substring(0,132));
                            s = s.substring(133);
                        } else {
                            results.add(s);
                            break;
                        }
                    }
                }
                br.close();
                p.waitFor();
                logger.info("Exit status: "+p.exitValue());
                return results;
            } catch (Exception e) {
                logger.error("Unexpected error executing cmd:"+e);
                e.printStackTrace();
                return e;
            }
        }
        
        public String toString() {
            StringBuffer sb = new StringBuffer(500);
            sb.append("{path:"+path+" (exists="+path.exists()+") cmd:"+cmd);
            return sb.toString();
        }
    }

    class TargetURL {
        public String url = null;
        public String params = null;
        public String method = "GET";
        public String user = null;
        public String pwd = null;
        
        public Object handleURL() {
            logger.info("Retrieving url: "+this);
		    try {
			    HttpRequester hr = new HttpRequester();
			    Map props = null;
                if (params!=null) props = HttpConverter.cvtURLStringToMap (params, "&");
			    hr.setRequest (url, method, props, user, pwd, null);
			    hr.connect();
                List results = new ArrayList(100);
			    String s;
			    while ((s = hr.readLine()) != null) {
//                    results.add(s);
                    while (true) {
                        if (s.length()>132) {
                            results.add(s.substring(0,132));
                            s = s.substring(133);
                        } else {
                            results.add(s);
                            break;
                        }
                    }
			    }
			    hr.disconnect();
                return results;
		    } catch (Exception e) {
			    e.printStackTrace();
                return e;                        
		    }	
        }

        public String toString() {
            return "{url:"+url+" params:"+params+" method:"+method+" user:"+user+" pwd:"+pwd+"}";
        }
    }

    
    
    /**
     * If you are having problems getting this to run, you can use this
     * static main method for testing purposes. You'll need to modify the
     * hardcoded values accordingly.
     */
    public static void main(String margs[]) {
        new BarracudaAdmin().runTest();
    }

    public void runTest() {
        TargetCommand tc = new TargetCommand();
/*        
        tc.path = new File("E:/WebProjects/lutris/cvs/Barracuda/src");
        tc.args = new String[] {"ant.bat","admin.sample1"};
        printTestResults(tc.handleCommand());
*/
        tc.path = new File("E:/WebProjects/lutris/cvs/Barracuda/src");
        tc.cmd = "E:/WebProjects/lutris/cvs/Barracuda/src/ant.bat admin.sample1";
        printTestResults(tc.handleCommand());
/*    
        TargetURL tu = new TargetURL();
        tu.url = "http://cryder:8080/manager/list";
        tu.params = null;
        tu.user = "admin";
        tu.pwd = "123123";
        printTestResults(tu.handleURL());

        tu.url = "http://cryder:8080/manager/reload?path=/examples";
        printTestResults(tu.handleURL());
*/
    }
    
    public void printTestResults(Object cmdResult) {
        System.out.println("Output...");
        if (cmdResult instanceof List) {
            List l = (List) cmdResult;
            for (int i=0; i<l.size(); i++) {
                System.out.println(l.get(i));
            }
        } else if (cmdResult instanceof Exception) {
            StringWriter sw = new StringWriter();
            ((Exception) cmdResult).printStackTrace(new PrintWriter(sw));
            System.out.println(sw.toString());
        } else {
            System.out.println(""+cmdResult);
        }
        System.out.println("");
    }

}