package org.enhydra.barracuda.contrib.dbroggisch.display.filters;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.*;
import org.apache.log4j.Logger;
import org.enhydra.barracuda.contrib.dbroggisch.display.*;

public class FilterFactory {
    private static Logger logger = Logger.getLogger(FilterFactory.class.getName());
	static Map s_filterMaps = new HashMap();
	static Map s_xml2filterMap = new HashMap();
	static Map s_classDescriptors = new HashMap();


	public static void addMapping(InputStream file) {
		try {
			FiltermappingUnmarshaller um = new FiltermappingUnmarshaller();
			Filtermapping fm = um.unmarshal(file, true);
			String xmlPBase = fm.getDtdpackage();
			if(xmlPBase != null && xmlPBase.trim().equals("")) {
				xmlPBase = null;								
			}
			List mapsets = fm.getMapsetList();
			if(logger.isDebugEnabled()) {
				logger.debug("XML package: " + xmlPBase);
				logger.debug("Found " + mapsets.size() + " mapsets.");
			}

			for(Iterator it = mapsets.iterator(); it.hasNext();) {
				Mapset mapset = (Mapset)it.next();
				String filterPBase = mapset.getXmlpackage();
				if(filterPBase != null && filterPBase.trim().equals("")) {
					filterPBase = null;								
				}
				if(logger.isDebugEnabled()) {
					logger.debug("Filter package: " + filterPBase);
				}

				List mapentries = mapset.getMapentryList();
				for(Iterator mpeIt = mapentries.iterator(); mpeIt.hasNext();) {
					// We want to get as much as possible mappings
					try {
						Mapentry mapentry = (Mapentry)mpeIt.next();
						if(logger.isDebugEnabled()) {
							logger.debug("Mapentry filterclass: " + mapentry.getFilterclass() + ", xmlclass: " + mapentry.getXmlclass());
						}
						Class filterClass = Class.forName(filterPBase + (filterPBase == null ? "" : ".") + mapentry.getFilterclass());
						Class xmlClass = Class.forName(xmlPBase + (xmlPBase == null ? "" : ".") + mapentry.getXmlclass());
						s_xml2filterMap.put(xmlClass, filterClass);
					} catch(ClassNotFoundException ex) {
						logger.error(ex.getMessage());
						ex.printStackTrace();
					}
				}
			}
		} catch(IOException ex) {
			logger.error(ex.getMessage());
			ex.printStackTrace();
		}
	}


	public static void loadFilters(InputStream file) 
		throws FilterException
	{
		try {
			s_filterMaps.clear();
			FilterfactoryUnmarshaller um = new FilterfactoryUnmarshaller();
			Filterfactory ff = um.unmarshal(file, true);
			List ml = ff.getFiltermapList();
			if(logger.isDebugEnabled()) {
				logger.debug("Found " + ml.size() + " mappings.");
			}
			// Create all FilterMaps
			for(Iterator it = ml.iterator(); it.hasNext();) {
				//				try {
				// First we have to look if our filtemap extends
				// an existing one.				
				Filtermap mp = (Filtermap)it.next();
				FilterMap fm;
				if(mp.getXmlextends() != null && 
				   !mp.getXmlextends().trim().equals("")) {
					if(s_filterMaps.get(mp.getXmlextends()) != null) {
						fm = (FilterMap)((FilterMap)s_filterMaps.get(mp.getXmlextends())).clone();
					} else {
						throw new FilterException("No such filtermap: " + mp.getXmlextends());
					}							
				} else {
					fm = new FilterMap();
				}
				fm.setName(mp.getName());
				// Handle the %extend.els;
				// (look into the filter.dtd for that)
				List removes = mp.getRemoveList();
				for(Iterator rit = removes.iterator(); rit.hasNext();) {
					Remove rem = (Remove)rit.next();
					if(logger.isDebugEnabled()) {
						logger.debug("Removing " + rem.getName());
					}
					fm.remove(rem.getName());
				}
				List remaps = mp.getRemapList();
				for(Iterator rit = remaps.iterator(); rit.hasNext();) {
					Remap rem = (Remap)rit.next();
					if(logger.isDebugEnabled()) {
						logger.debug("Remapping " + rem.getFrom() + " to " + rem.getTo());
						logger.debug("From exists: " + fm.get(rem.getFrom()) != null ? "yes" : "no");
					}

					fm.put(rem.getTo(), fm.get(rem.getFrom()));
				}
				List copies = mp.getCopyList();
				Map copy_map = new HashMap();
				for(Iterator rit = copies.iterator(); rit.hasNext();) {
					Copy copy = (Copy)rit.next();
					copy_map.put(copy.getFrom(), copy.getTo());
				}
				fm.setCopies(copy_map);

				// Add the declared filters
				List filters = mp.getFilterList();
				if(logger.isDebugEnabled()) {
					logger.debug("Adding " + filters.size() + " filters ");
				}

				for(Iterator rit = filters.iterator(); rit.hasNext();) {
					org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Filter f = (org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Filter)rit.next();
					Object conf = getChild(f);
					org.enhydra.barracuda.contrib.dbroggisch.display.filters.Filter filter = createFilter(conf.getClass());
					if(logger.isDebugEnabled()) {
						logger.debug("Adding filter " + f.getName() + ", Type is: " + filter.getClass());
					}
					// Assign the configured filter - it might be 
					// a different filter.
					filter = filter.configure(conf);
					fm.put(f.getName(), filter);
				}

				// Finally - we save the filtermap
				if(logger.isDebugEnabled()) {
					logger.debug("Adding filtermap " + fm.getName());
				}

				s_filterMaps.put(fm.getName(), fm);
				// 					}
// 				}
// 				catch(FilterException ex) {
// 					logger.error(ex.getMessage());
// 					ex.printStackTrace();
// 				}

			}
			ml = ff.getClassdescriptorList();
			if(logger.isDebugEnabled()) {
				logger.debug("Found " + ml.size() + " classdescriptors.");
			}
			// Create all ClassDescriptors
			for(Iterator it = ml.iterator(); it.hasNext();) {
				Classdescriptor cd = (Classdescriptor)it.next();
				try {
					Class klass = Class.forName(cd.getClassname());
					ClassDescriptor clsDesc;
					if(cd.getXmlextends() != null &&
					   !cd.getXmlextends().trim().equals("")) {
						Class extendee = Class.forName(cd.getXmlextends());
						if(s_classDescriptors.get(extendee) != null) {
							if(logger.isDebugEnabled()) {
								logger.debug("ClassDesscriptor extends " + extendee);
							}
							clsDesc = (ClassDescriptor)((ClassDescriptor)s_classDescriptors.get(extendee)).clone();
						} else {
							throw new FilterException("No such classdescriptor: " + cd.getXmlextends());
						}
					} else {
						clsDesc = new ClassDescriptor();
					}
					// Resolve the maprefs
					List maprefs = cd.getMaprefList();
					
					for(Iterator mit = maprefs.iterator(); mit.hasNext();) {
						Mapref mr = (Mapref)mit.next();
						FilterMap fm = (FilterMap)s_filterMaps.get(mr.getRef());
						clsDesc.put(mr.getChannel(), fm);
					}
					// Finally, put the classdescriptor into the map
					if(logger.isDebugEnabled()) {
						logger.debug("Adding ClassDescriptor for " + klass);
					}
					s_classDescriptors.put(klass, clsDesc);
				} catch(ClassNotFoundException ex) {
 					logger.error(ex.getMessage());
 					ex.printStackTrace();
					throw new FilterException("No class " + cd.getClassname() + " found");
 				}				
			}

		} catch(IOException ex) {
			logger.error(ex.getMessage());
			ex.printStackTrace();
		} 
	}

	public static Map getFilterMap(String name) {
		return (Map)s_filterMaps.get(name);
	}

	protected static Filter createFilter(Class xmlClass) {
		if(logger.isDebugEnabled()) {
			logger.debug("Creating Filter for " + xmlClass.getName());
		}

		Class [] iFaces = xmlClass.getInterfaces();

		Class filterClass = (Class)s_xml2filterMap.get(xmlClass);
		if(filterClass == null) {
			for(int i = 0; i < iFaces.length; i++) {
				if(logger.isDebugEnabled()) {
					logger.debug("Loking up class: " + iFaces[i].getName());
				}

				filterClass = (Class)s_xml2filterMap.get(iFaces[i]);
				if(filterClass != null)
					break;
			}
		}
		if(filterClass != null) {
			if(logger.isDebugEnabled()) {
				logger.debug("Filterclass found: " + filterClass.getName());
			}
			try {
				return (Filter)filterClass.newInstance();
			} catch(InstantiationException ex) {
				logger.error(ex.getMessage());
				ex.printStackTrace();
			} catch(IllegalAccessException ex) {
				logger.error(ex.getMessage());
				ex.printStackTrace();
			}
		} else {
			logger.warn("No Filterclass found for: " + xmlClass.getName());
		}
		return null;
	}

	private static List getters;
	private static Object getChild(Object obj) {
		try {									
			if(getters == null) {
				getters = new ArrayList();
				Method [] mths = obj.getClass().getMethods();
				Set allowedClasses = s_xml2filterMap.keySet();
				if(logger.isDebugEnabled()) {
					logger.debug("Allowed classes: " + allowedClasses);
				}
				for(int i = 0; i < mths.length; i++) {
					Method m = mths[i];
					Class returnType = m.getReturnType();								
											
					if(logger.isDebugEnabled()) {
						logger.debug("handling method " + m.getName() + " with type " + returnType.getName());
					}
					if(m.getName().startsWith("get")  
					   && allowedClasses.contains(returnType)) {
						getters.add(m);
					}
				}
			}
			for(Iterator it = getters.iterator(); it.hasNext();) {
				Method m = (Method)it.next();
				Object res = m.invoke(obj, null);
				if(res != null) {
					return res;
				}
			}
						
		} catch(InvocationTargetException ex) {
			logger.error(ex.getMessage());
		} catch(IllegalAccessException ex) {
			logger.error(ex.getMessage());
		}
		return null;
	}


	public static List accumulateChilds(Object obj) {
		List res = new ArrayList();
		try {									
			Method [] mths = obj.getClass().getMethods();
			for(int i = 0; i < mths.length; i++) {
				Method m = mths[i];
				Class returnType = m.getReturnType();											
				if(logger.isDebugEnabled()) {
					logger.debug("handling method " + m.getName() + " with type " + returnType.getName());
				}
				if(m.getName().startsWith("get")  
				   && returnType.equals(List.class)) {
					res.addAll((List)m.invoke(obj, null));
				}
			}
		} catch(InvocationTargetException ex) {
			logger.error(ex.getMessage());
		} catch(IllegalAccessException ex) {
			logger.error(ex.getMessage());
		}
		return res;
	}


	private static class FilterMap extends HashMap {
		String _name;
		Map _copies;

		public void setCopies(Map copies) {
			_copies = copies;
		}

		public Map getCopies() {
			return _copies;
		}

		public String getName() {
			return _name;
		}
		public void setName(String name) {
			_name = name;
		}

		public Object clone() {
			FilterMap fm = new FilterMap();
			fm.setName(_name);
			fm.putAll(this);
			return fm;
		}
	}

	private static class ClassDescriptor extends HashMap implements Cloneable {
		Class _class;
		FilterMap _default;

		
		public Object put(Object key, Object value) {
			if(logger.isDebugEnabled() && value != null) {
				logger.debug("Setting FilterMap " + ((FilterMap)value).getName());
			}

			if(key != null) {
				if(logger.isDebugEnabled() && value != null) {
					logger.debug("... as class " + key);
				}
				return super.put(key, value);
			} else {
				if(logger.isDebugEnabled() && value != null) {
					logger.debug("... as default");
				}
				Object old_d = _default;
				_default = (FilterMap)value;
				return old_d;
			}
		}



		public Class getMappedClass() {
			return _class;
		}

		public void setMappedClass(Class klass) {
			_class = klass;
		}
		
		public FilterMap getDefault() {
			return _default;
		}

		public void setDefault(FilterMap ddefault) {
			_default = ddefault;
		}

		public Object clone()
		{
 			ClassDescriptor cd = new ClassDescriptor();
 			cd.setMappedClass(_class);
 			cd.setDefault(_default);
			cd.putAll(this);
 			return cd;
		}

	}


	public static LightweightTemplateModel filter(Object obj, 
									FilterContext ctx) 
		throws FilterException
	{
		if(obj != null) {

			LightweightTemplateModel ltm = ReflectionMapper.map(obj);
			if(logger.isDebugEnabled()) {
				logger.debug("Mapping");
				logger.debug(ltm != null ? "...was sucsssful" : "...wasn't sucsssful");
			}
			ClassDescriptor clsDesc = (ClassDescriptor)s_classDescriptors.get(obj.getClass());
			// If we have a class descriptor, try to fetch a FilertMap
			// Otherwise simlply return the model
			if(clsDesc != null) {
				if(logger.isDebugEnabled()) {
					logger.debug("Found ClassDescriptor for " + obj.getClass());
				}
				FilterMap fm = (FilterMap)clsDesc.get(ctx.getChannel());
				if(fm == null) {
					if(logger.isDebugEnabled()) {
						logger.debug("No FilterMap for channel " + ctx.getChannel() + ", using default");
					}
					fm = clsDesc.getDefault();
				}
				if(fm != null) {
					ltm = filter(ltm, fm, ctx);
				} else {
					if(logger.isDebugEnabled()) {
						logger.debug("No FilterMap found!");
					}					
				}
			} else {
				logger.warn("No classmapping for " + obj.getClass());
			}
			return ltm;
		}
		return null;
	}

	public static LightweightTemplateModel filter(LightweightTemplateModel ltm, 
									String fmName,
									FilterContext ctx) 
		throws FilterException
	{
		FilterMap fm = (FilterMap)s_filterMaps.get(fmName);
		if(fm == null) {
			throw new FilterException("No such FilterMap " + fmName);
		}
		return filter(ltm, fm, ctx);
	}
	
	/**
	 * 
	 *
	 * @param ltm a <code>LightweightTemplateModel</code> value
	 * @param filterMap The FilterMap to apply
	 * @param ctx the FilterContext to be used
	 * @return a <code>LightweightTemplateModel</code>
	 * @exception FilterException if an error occurs
	 */
	public static LightweightTemplateModel filter(LightweightTemplateModel ltm, 
									FilterMap fm,
									FilterContext ctx) 
		throws FilterException
	{
		HashMapModel res = new HashMapModel();
		res.setName(ltm.getName());
		// First, execute the copies
		Map copies = fm.getCopies();
		for(Iterator cp = copies.entrySet().iterator(); cp.hasNext();) {
			Map.Entry entry = (Map.Entry)cp.next();
			if(logger.isDebugEnabled()) {
				logger.debug("Copyiing " + entry.getKey() + " to " + entry.getValue());
			}
			res.setItem((String)entry.getValue(), ltm.getItem((String)entry.getKey()));			
		}
		// Filter the values in ltm
		for(Iterator it = ltm.keySet().iterator(); it.hasNext();) {
			String key = (String)it.next();
			Filter f = (Filter)fm.get(key);
			if(f != null) {
				if(logger.isDebugEnabled()) {
					logger.debug("Filtering " + key);
				}
				res.setItem(key, f.filter(ltm.getItem(key), ctx));
			} else {
				if(logger.isDebugEnabled()) {
					logger.debug("No filtering for " + key);
				}
				res.setItem(key, ltm.getItem(key));
			}
		}
		// Now perform filtering on the copied values
		for(Iterator it = copies.keySet().iterator(); it.hasNext();) {
			String key = (String)it.next();
			Filter f = (Filter)fm.get(key);
			if(f != null) {
				res.setItem(key, f.filter(res.getItem(key), ctx));
			}
		}		
		return res;
	}
}
