package org.enhydra.barracuda.contrib.dbroggisch.display.filters;

import java.util.*;
import java.io.*;
import org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Lookup;
import org.enhydra.barracuda.contrib.dbroggisch.display.filters.dtd.Entry;
import org.apache.log4j.Logger;

/* This class uses a given Map to lookup the 
 * objects meaning. 
 *
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public class LookupFilter implements Filter {
    protected static Logger logger = Logger.getLogger(LookupFilter.class.getName());

    private Map lookupMap;

    public LookupFilter() {
		this(null);
    }

    public LookupFilter(Map lookupMap) {
		this.lookupMap = lookupMap;
    }

    /** Looks up the given key. If the key doesn't exist, <code>null</code>
     * returned. If the lookupMap is <code>null</code>, the key itself will be returned.
     *
     * @param key The key to lookup.
     * @return the mapped objet.
     */
    public Object filter(Object key, FilterContext context)
		throws FilterException 
    {
		if(key != null && lookupMap != null) {
			return lookupMap.get(key);
		}
		return key;
    }

    public Filter configure(Object obj)
		throws FilterException 
    {
		lookupMap = new HashMap();
		try {
			Lookup xmlObj = (Lookup)obj;
			List entries = xmlObj.getEntryList();
			for(Iterator it = entries.iterator(); it.hasNext();) {
				Entry entry = (Entry)it.next();
				Object key;
				if(entry.getType().equals("Integer")) {
					key = new Integer(Integer.parseInt(entry.getKey()));
				} else if(entry.getType().equals("Boolean")) {
					key = Boolean.valueOf(entry.getKey());
				} else if(entry.getType().equals("Class")) {
					try {
						key = Class.forName(entry.getKey());
					} catch(ClassNotFoundException ex) {
						StringWriter sw = new StringWriter();
						ex.printStackTrace(new PrintWriter(sw));
						logger.error(sw.toString());
						throw new FilterException(ex.getMessage());
					}
				} else {
					key = entry.getKey();
				}
				lookupMap.put(key, entry.getValue());
			}
		} catch(ClassCastException ex) {						
			ex.printStackTrace();						
		}
		return this;
    }
}
