package org.enhydra.barracuda.contrib.dbroggisch.page;

import org.enhydra.barracuda.core.event.DefaultBaseEventListener;
import org.enhydra.barracuda.core.event.ControlEventContext;
import org.enhydra.barracuda.core.event.EventException;
import org.enhydra.barracuda.core.event.BaseEvent;
import org.enhydra.barracuda.core.event.InterruptDispatchException;

import java.io.*;

import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

public abstract class PageEventListener extends DefaultBaseEventListener {

    final private static Logger logger = Logger.getLogger(PageEventListener.class.getName());

    public final void handleControlEvent(ControlEventContext context)
                                             throws EventException,
                                                    javax.servlet.ServletException,
                                                    java.io.IOException {

	    HttpServletRequest req = context.getRequest();
    	HttpSession session = req.getSession(false);
	
	    if (session == null) {
	        if (logger.isDebugEnabled()) logger.error("No session for page-handling found!");
	    
	        // TODO: Errorhandling
 	        //return;
	    }
	
    	Page page = null;
    	if (session!=null) {
    	    page = (Page)session.getAttribute("PAGE");
    	}
    	if (page == null) {
    	    page = (Page)context.getState("PAGE");
    	}
    	
    	BaseEvent event = context.getEvent();
    	if (page == null) {
    	    if (logger.isDebugEnabled()) logger.debug("Creating fresh PAGE for event " + context.getEvent().getClass().getName());
    	    
    	    page = PageFactory.createPage(event);
    	    if (page != null) {
    		    page.createDefaultModels(context);
    	    }
    	} else {
    	    Page newPage = null;
    	    if (logger.isDebugEnabled()) logger.debug("Creating page to handover for event " + context.getEvent().getClass().getName());
    	    
    	    if (!page.getClass().equals(PageFactory.getPageClass(event))) {
        		newPage = PageFactory.createPage(event);
        		if (newPage != null) {
        		    if (logger.isDebugEnabled()) logger.debug("Page differs, handing over");
        		    newPage.handOver(context, page);
        		    page = newPage;
        		}		    
    	    }
    	}
    
    	if (page instanceof ContextPage || session==null) {
    	    if (session != null) {
    		    session.removeAttribute("PAGE");
    	    }
    	    context.putState("PAGE", page);
    	} else {
    	    session.setAttribute("PAGE", page);
    	}
    
    	try {
    	    handleControlEvent(context, page);
    	} catch (ClassCastException ex) {
    	    logger.error("ClassCastException occured in handleControlEvent()");
    	    
    	    StringWriter sw = new StringWriter();
    	    ex.printStackTrace(new PrintWriter(sw));
    	    logger.error(sw.toString());
    	}
    }

    public abstract void handleControlEvent(ControlEventContext context, Page page)
        throws EventException, javax.servlet.ServletException, java.io.IOException;


}
