/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: DefaultViewContext.java,v 1.6 2001/10/18 17:58:35 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.w3c.dom.*;

import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.view.*;

/**
 * This class provides the default implementation of a ViewContext. A View
 * Context basically exists to provide the component models with the 
 * information they need to actually pass back the proper data to the 
 * component.
 */
public class DefaultViewContext implements ViewContext {

    //private vars
    private StateMap statemap = new DefaultStateMap();

    //--------------- Constructors -------------------------------
    /**
     * Create a DefaultViewContext
     */
    public DefaultViewContext() {}
    
    /**
     * Create a DefaultViewContext for a specific ViewCapabilities obj
     */
    public DefaultViewContext(ViewCapabilities ivc, HttpServletRequest ireq, HttpServletResponse iresp) {
        this.putState(ViewContext.VIEW_CAPABILITIES, ivc);
        this.putState(ViewContext.REQUEST, ireq);
        this.putState(ViewContext.RESPONSE, iresp);
    }
    
    /**
     * Create a DefaultViewContext for a specific EventContext
     */
    public DefaultViewContext(ViewEventContext ivec) {
        this.putState(ViewContext.VIEW_CAPABILITIES, ivec.getViewCapabilities());
        this.putState(ViewContext.EVENT_CONTEXT, ivec);
        this.putState(ViewContext.REQUEST, ivec.getRequest());
        this.putState(ViewContext.RESPONSE, ivec.getResponse());
    }



    //--------------- DefaultViewContext -------------------------
    /**
     * Get the underlying ViewCapabilities object
     *
     * @return the underlying ViewCapabilities object
     */
    public ViewCapabilities getViewCapabilities() {
        return (ViewCapabilities) this.getState(ViewContext.VIEW_CAPABILITIES);
    }

    /**
     * Get the underlying EventContext object
     *
     * @return the underlying EventContext object
     */
    public EventContext getEventContext() {
        return (EventContext) this.getState(ViewContext.EVENT_CONTEXT);
    }

    /**
     * Get the underlying ElementFactory object (if it exists)
     *
     * @return the underlying ElementFactory object (if it exists)
     */
    public ElementFactory getElementFactory() {
        return (ElementFactory) this.getState(ViewContext.ELEMENT_FACTORY);
    }
    
    /**
     * Get the underlying template Node (if it exists)
     *
     * @return the underlying template Node (if it exists)
     */
    public Node getTemplateNode() {
        return (Node) this.getState(ViewContext.TEMPLATE_NODE);
    }

    /**
     * Get the underlying HttpServletRequest
     *
     * @return the underlying HttpServletRequest
     */
    public HttpServletRequest getRequest() {
        return (HttpServletRequest) this.getState(ViewContext.REQUEST);
    }

    /**
     * Get the underlying HttpServletResponse
     *
     * @return the underlying HttpServletResponse
     */
    public HttpServletResponse getResponse() {
        return (HttpServletResponse) this.getState(ViewContext.RESPONSE);
    }


    
    //-------------------- StateMap ------------------------------
    /**
     * set a property in this StateMap
     *
     * @param key the state key object
     * @param val the state value object
     */    
    public void putState(Object key, Object val) {
        statemap.putState(key,val);
    }
    
    /**
     * get a property in this StateMap
     *
     * @param key the state key object
     * @return the value for the given key
     */    
    public Object getState(Object key) {
        return statemap.getState(key);
    }
    
    /**
     * remove a property in this StateMap
     *
     * @param key the key object
     * @return the object which was removed
     */    
    public Object removeState(Object key) {
        return statemap.removeState(key);
    }
    
    /**
     * get a list of the keys for this StateMap
     *
     * @return a list the keys for this StateMap
     */    
    public List getStateKeys() {
        return statemap.getStateKeys();
    }
    
    /**
     * get a copy of the underlying Map
     *
     * @return a copy of the underlying state Map
     */    
    public Map getStateValues() {
        return statemap.getStateValues();    
    }
}