/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EnabledHelper.java,v 1.3 2001/09/25 19:05:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.comp.renderer.html;

import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.renderer.*;

/**
 * This class makes sure that components are rendered properly
 * for enabled or disabled states
 */
public class EnabledHelper extends DOMComponentRenderer {

    /**
     * For given elements, set their disabled attribute
     */
    public void setEnabled(Node node, boolean enabled) {

        //..HTMLAnchorElement - clear the "href" and "target" attributes
        if (node instanceof HTMLAnchorElement) {
            Element el = (Element) node;
            if (!enabled) el.removeAttribute("href");
            if (!enabled) el.removeAttribute("target");

        //..HTMLButtonElement - set the "disabled" attribute
        } else if (node instanceof HTMLButtonElement) {
            ((HTMLButtonElement) node).setDisabled(!enabled);

        //..HTMLInputElement - set the "disabled" attribute
        } else if (node instanceof HTMLInputElement) {
            ((HTMLInputElement) node).setDisabled(!enabled);

        //..HTMLOptGroupElement - set the "disabled" attribute
        } else if (node instanceof HTMLOptGroupElement) {
            ((HTMLOptGroupElement) node).setDisabled(!enabled);

        //..HTMLOptionElement - set the "disabled" attribute
        } else if (node instanceof HTMLOptionElement) {
            ((HTMLOptionElement) node).setDisabled(!enabled);

        //..HTMLSelectElement - set the "disabled" attribute
        } else if (node instanceof HTMLSelectElement) {
            ((HTMLSelectElement) node).setDisabled(!enabled);

        //..HTMLStyleElement - set the "disabled" attribute
        } else if (node instanceof HTMLStyleElement) {
            ((HTMLStyleElement) node).setDisabled(!enabled);

        //..HTMLTextAreaElement - set the "disabled" attribute
        } else if (node instanceof HTMLTextAreaElement) {
            ((HTMLTextAreaElement) node).setDisabled(!enabled);
        }

    }
    
}