/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventDispatcher.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.io.*;
import java.util.*;

/**
 * This interface defines the methods needed to implement an 
 * EventDispatcher
 */
public interface EventDispatcher {

    /**
     * Dispatch a queue of events. The incoming queue must be an instance 
     * of DefaultDispatchQueue or an EventException will be thrown. This 
     * means that if you're going to provide a custom event broker, you
     * may need to provide a custom dispatcher as well.
     *
     * @param eb the event broker to be used to match events to listeners
     * @param ieventQueue the event queue to be dispatched
     * @throws EventException
     */
    public void dispatchEvent(EventBroker eb, EventContext context) throws EventException;

    /** @link dependency */
    /*#DispatchQueue lnkDispatchQueue;*/
}
