/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: EventPool.java,v 1.5 2001/09/25 19:05:29 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event;

import java.util.*;

/**
 * This indterface defines the methods needed to implement an 
 * EventPool
 */
public interface EventPool {

    /**
     * check out an event from the EventPool.
     *
     * @param event the class of event we are interested in checking out
     * @return the checked out event
     * @throws NoAvailableEventsException if there are no available events in the queue
     * @throws InvalidClassException if the event class is not valid
     */    
    public BaseEvent checkoutEvent(Class event) throws NoAvailableEventsException, InvalidClassException;

    /**
     * check the event back in, allowing someone 
     * else to have access to it.
     *
     * @param event the event we're releasing our lock on
     */    
    public void releaseEvent(BaseEvent event);

    /**
     * Cleanup any locked events which weren't released 
     * (they should all be). You should not ever really need
     * to run this method. It will get invoked automatically
     * when the cleaner-upper runs
     */
    public void cleanupLockedEvents();

    //lb_032801 - Added for patch submitted by Larry Brasfield 
    //[larry.brasfield@theplatform.com] so JVM can exit once 
    //the servlet's destroy() method is called.   
    /**
     * Shutdown the event pool
     */
    public void shutdown();
}
