/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: BlockIterateHandler.java,v 1.8 2002/06/17 14:33:57 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.event.helper;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.helper.servlet.*;
import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.exceptions.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.core.util.http.*;
import org.apache.log4j.*;

/**
 * 
 */
public abstract class BlockIterateHandler extends DefaultBaseEventListener {

    //public vars...eventually, these should probably be final
    public static Logger localLogger = Logger.getLogger(BlockIterateHandler.class.getName());
    protected static final String NODE_ITERATOR_FLAG = "Dir::Block_Iterate.".toLowerCase();

    //other vars
//    protected String spaces = "                                                                              ";
    protected ViewContext vc = null;
    protected BlockIterator bi = null;
    
    /**
     * Public noargs constructor
     */
//    public BlockIterateHandler() {}


    //-------------------- BlockIterateHandler -------------------
    public void setViewContext(ViewContext ivc) {
        vc = ivc;
    }
    
    public ViewContext getViewContext() {
        return vc;
    }
    
    public void updateModelInTemplate(TemplateModel model) {
        if (bi!=null) bi.updateModelInTemplate(model);
    }
    
    public DOMWriter getDOMWriter() {
        DefaultDOMWriter dw = new DefaultDOMWriter();
        dw.setLeaveWriterOpen(true);
        return dw;
    }

    /**
     * you can override this method to handle any initializion 
     * needs for the handler. Its invoked after the view context info
     * has been set up
     */
    public void initHandler() {
    }

    public abstract Class getTemplateClass();

    public abstract BlockIterator getIterator(String key);

    /**
     * Handle the ViewEvent
     */
    public void handleViewEvent(ViewEventContext vec) throws EventException, ServletException, IOException {
        long bmillis = 0;
//            long smillis = 0;
//            long emillis = 0;
//            long elapsed1 = 0;
//            long elapsed2 = 0;
        if (localLogger.isInfoEnabled()) bmillis = System.currentTimeMillis();
        if (localLogger.isInfoEnabled()) localLogger.info("Handling ViewEvent in "+this);

        try {
            //start by figuring out the ViewCapabilities
            if (localLogger.isDebugEnabled()) localLogger.debug("Create the ViewContext");
            setViewContext(new DefaultViewContext(vec));
            HttpServletResponse resp = vec.getResponse();
            PrintWriter out = resp.getWriter();

            //give the handler a chance to initialize
            initHandler();

            //load the localized DOM template
            if (localLogger.isDebugEnabled()) localLogger.debug("Getting the DOM Template");
//csc_061702.1            Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass());
            Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass(), vec.getViewCapabilities()); //csc_061702.1

            //csc_061202.1 - added    
            //set the caching hdrs (this will allow the static resources to be cached by the browser)
            resp.setHeader("Cache-Control","max-age=0");
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());

            //set the content header & then process the page
            boolean isHtml = false;
            String responseType = "text/xml";
            if (page instanceof HTMLDocument) {
                responseType = "text/html";
                isHtml = true;
            }
            resp.setContentType(responseType); 
            if (localLogger.isDebugEnabled()) localLogger.debug("Printing page (content type: "+responseType+")");
            Element root = page.getDocumentElement();
//System.out.println("Here 0");            
            printNode(page, out, 0, isHtml);
//System.out.println("Here 1");            
            out.close();
//System.out.println("Here 2");            

            if (localLogger.isInfoEnabled()) localLogger.info("ViewEvent handled! (rendered in "+(System.currentTimeMillis()-bmillis)+" millis)");
        
        } catch (RenderException e) {
//System.out.println("Here 2.err");            
            //if we get an EventException, handle it
            if (localLogger.isInfoEnabled()) localLogger.info("Unexpected RenderException:"+e);
            throw new EventException ("Unexpected RenderException:"+e, e);
/*
        } catch (EventException e) {
System.out.println("Unexpected exception:"+e);            
            throw e;
        } catch (ServletException e) {
System.out.println("Unexpected exception:"+e);            
            throw e;
*/            
        } catch (IOException e) {
//System.out.println("Unexpected exception:"+e);            
            localLogger.warn("Unexpected IOException:"+e);            
            e.printStackTrace();
            throw e;

        } catch (RuntimeException e) {
            localLogger.warn("Unexpected RuntimeException:"+e);            
            e.printStackTrace();
            throw e;
            
        } finally {
//System.out.println("Here 3");            
            //cleanup
            vc = null;
            bi = null;
            
            //as a final measure, ask the system to invoke gc()
            System.gc();
//System.out.println("Here 99");            
        }

    }

    public void printNode(Node node, PrintWriter out, int depth, boolean isHtml) throws RenderException, IOException {
//        if (depth<0) depth = 0;
//        if (depth>25) depth = 25;
//        String inset = spaces.substring(0,depth*3);

        //element
        if (node instanceof Element) {
            Element el = (Element) node;
            //first of all, look for the Node_Iterator directive in the attributes
            String blockIteratorName = null;
            Attr attr = el.getAttributeNode("class");
            if (attr!=null) {
                String value = attr.getValue();
                int spos = value.toLowerCase().indexOf(NODE_ITERATOR_FLAG);
                if (spos>-1) {
                    int lpos = value.length();
                    int epos = value.indexOf(" ", spos+NODE_ITERATOR_FLAG.length());
                    if (epos<0 || epos>lpos) epos = lpos;
                    blockIteratorName = value.substring(spos+NODE_ITERATOR_FLAG.length(), epos);
                    value = value.substring(0,spos)+(epos<lpos ? value.substring(epos+1, lpos) : "");
                    attr.setValue(value);
//System.out.println("Here 0:"+blockIteratorName+" newValue:"+attr);            
                }
            }

            //if we have a blockIteratorName, use that
            if (blockIteratorName!=null) {
                if (localLogger.isDebugEnabled()) localLogger.debug("Found Block_Iterator directive:"+blockIteratorName);
                int cntr = 0;
                bi = getIterator(blockIteratorName);
                if (bi!=null) {
                    try {
                        while (bi.hasNext()) {
                            //get the new node (which the component has already rendered into)
                            Node templateNode = node.cloneNode(true);
    //                        vc.putState(ViewContext.TEMPLATE_NODE, templateNode);
//                            bi.setTemplateNode(templateNode);
                            Node newNode = bi.next(getViewContext(), templateNode);
                            cntr++;
                            if (localLogger.isDebugEnabled()) localLogger.debug("Got block "+cntr+", node: "+newNode);
    //System.out.println("newNode:"+newNode);
                            if (newNode==null) continue;
                        
                            //now render this node to the output stream
                            getDOMWriter().write(newNode, out);
                            if (localLogger.isDebugEnabled()) localLogger.debug("Successfully rendered node!");
                        }
                    } finally {
                        bi = null;
                    }
                } else {
                    if (localLogger.isDebugEnabled()) localLogger.warn("Failed to locate corresponding BlockIterator class!");
                    out.println("");
                    out.println("");
                    out.println("<!-- Missing Iterator: "+blockIteratorName+" -->");
                    out.println("");
                }
//System.out.println("Here 1:");

            //otherwise, just print the tag and attributes            
            } else {
                out.print("<"+el.getTagName());
                String sep = " ";
                NamedNodeMap nnm = el.getAttributes();
                for (int i=0, max=nnm.getLength(); i<max; i++) {
                    attr = (Attr) nnm.item(i);
                    out.print(sep+attr);
                }
                out.print(">");
        
                //print the children
                printChildNodes(node, out, depth, isHtml);
        
                //print the closing tag (if it's not forbidden, as the given html tags are)
                String tag = el.getTagName().toLowerCase();
                if (!isHtml || 
                   (!tag.equals("area") && !tag.equals("base") && !tag.equals("basefont") &&
                    !tag.equals("br") && !tag.equals("col") && !tag.equals("frame") &&
                    !tag.equals("hr") && !tag.equals("image") && !tag.equals("input") &&
                    !tag.equals("isindex") && !tag.equals("link") && !tag.equals("meta") &&
                    !tag.equals("param"))) {
                    out.print("</"+el.getTagName()+">");
                }
            }
        
        //character data
        } else if (node instanceof CharacterData) {
            if (node instanceof Comment) out.print("<!-- ");
            out.print(((CharacterData) node).getData());
            if (node instanceof Comment) out.print("-->");

        //node with child nodes        
        } else if (node.hasChildNodes()) {
            printChildNodes(node, out, depth, isHtml);

        //anything else
        } else {        
            System.out.println("Unhandled element:"+node.getClass().getName());
            org.enhydra.barracuda.core.util.data.Collections.printStackTrace(Classes.getAllInterfaces(node));
        }
    }

    public void printChildNodes(Node node, PrintWriter out, int depth, boolean isHtml) throws RenderException, IOException {
        Node child = node.getFirstChild();
        if (child==null) return;
        do {
            printNode(child, out, depth+1, isHtml);
            child = child.getNextSibling();
        } while (child!=null);
    }
}
