/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: NotNullValidator.java,v 1.11 2002/05/31 14:48:32 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.util.*;

import org.enhydra.barracuda.core.forms.*;
import org.enhydra.barracuda.core.util.*;

/**
 * This validator ensures that a form element is not null or blanks.
 */
public class NotNullValidator extends DefaultFormValidator {

    /**
     * Public constructor.
     *
     * @param errmsg the message associated with this error
     */
    public NotNullValidator() {
        super();
    }

    /**
     * Public constructor.
     *
     * @param errorMessage the message associated with this error
     */
    public NotNullValidator(String ierrorMessage) {
        super(ierrorMessage);
    }

    /**
     * Validate a FormElement to make sure that it is not Null or blanks.
     *
     * @param element the form element to be validated
     * @param element the form element that contains the val
     *        to validate elements by comparing them with other elements)
     * @param deferExceptions do we want to deferValidation exceptions
     *        and attempt to validate all elements so that we can process
     *        all the exceptions at once
     * @throws ValidationException if the element is not valid
     */
    public void validateFormElement(Object val, FormElement element, boolean deferExceptions) throws ValidationException {
        if (localLogger.isInfoEnabled()) localLogger.info("val="+val+" "+(val==null ? "true" : "false"));
        // ilc_022102.1_start
        // use isNull method to insure consistancy
        // if (val==null || val.toString().trim().length()<1) {
        if (this.isNull(val, element)) {
        // ilc_022102.1_end
            throw this.generateException(element, deferExceptions, "Value must be non-null or non-blanks");
        }
    }

}
