/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: LightweightURL.java,v 1.1 2001/10/03 21:25:20 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.http;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * A lightweight URL class
 */
public class LightweightURL {
    String url = null;
    String base = null;
    String param = null;

    /**
     * Create a lightweight url
     *
     * @param url a URL string
     */
    public LightweightURL(String iurl) {
        url = iurl;
    }

    /**
     * Return the whole URL string 
     * (ie. "http://foo.com?foo1=blah1&foo2=blah2")
     */
    public String getURLStr() {
        if (url==null) return "";
        return url;
    }

    /**
     * Return the base part of the URL (everything up to the ?)
     * (ie. "http://foo.com")
     */
    public String getBaseStr() {
        if (url==null) return "";
        if (base==null) {
            int epos = url.indexOf("?");
            if (epos>-1) base = url.substring(0,epos);
            else base = url;
        }
        return base;
    }

    /**
     * Return the whole URL string (from the ? to the end)
     * (ie. "?foo1=blah1&foo2=blah2")
     */
    public String getParamStr() {
        if (url==null) return "";
        if (param==null) {
            int spos = url.indexOf("?");
            if (spos>-1) param = url.substring(spos);
            else param = "";
        }
        return param;
    }
    
    /**
     * Return a String representation of the URL
     */
    public String toString() {
        return url;
    }
}    
