/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: URLRewriter.java,v 1.5 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.http;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.event.*;
import org.apache.log4j.*;

/**
 * This just contains some Servlet utility routines
 */
public class URLRewriter {

    /**
     * This option is used to control whether or not URL rewriting occurs. It 
     * exists to handle bugs in Enhydra 3.x in which invoking the URL rewriting
     * methods causes a bogus value to be returned. If you are using Enhydra 3.x,
     * you will want to set this value to false (you can do this without coding
     * changes via the your assembler xml file). To see whether or not your
     * appserver correctly implements URL rewriting, look at 
     * http://localhost/Barracuda/RedirectEx1
     */
//    public static boolean REWRITE_URLS = true;
    public static boolean REWRITE_URLS = false;            //need it false if we're running against Enhydra 3.x

    //-------------------- URL Rewriting -------------------------
    /**
     * Encode a URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param vc the ViewContext
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeURL(ViewContext vc, String url) {
        if (vc!=null) {
//csc_101701.1_start
/*        
            EventContext ec = vc.getEventContext();
            if (ec!=null && ec instanceof ViewEventContext) {
                ViewEventContext vec = (ViewEventContext) ec;
                return encodeURL(vec.getRequest(), vec.getResponse(), url);
            } 
*/            
            return encodeURL(vc.getRequest(), vc.getResponse(), url);
//csc_101701.1_end
        }
        return url;
    }
    
    /**
     * Encode a URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeURL(HttpServletRequest req, HttpServletResponse resp, String url) {
//csc_101701.1_start - This does not seem to work...the 
//isRequestedSessionIdValid() is always returning false in Tomcat 3.3...
//        if (REWRITE_URLS &&
//            req.isRequestedSessionIdValid() &&
//            !req.isRequestedSessionIdFromCookie()) return resp.encodeURL(url);
//        else return url;

        //This check basically says that if we want to rewrite URLs AND we have a
        //session AND the session id didn't come from a cookie, then go ahead and 
        //encode the url
        HttpSession session = req.getSession(false);
//System.out.println("Here1: REWRITE_URLS="+REWRITE_URLS+" session="+session+" !req.isRequestedSessionIdFromCookie()="+!req.isRequestedSessionIdFromCookie());
        if (REWRITE_URLS &&
            session!=null &&
            !req.isRequestedSessionIdFromCookie()) {
//System.out.println("Encoding URL! "+resp.encodeURL(url));            
            return resp.encodeURL(url);
            
        //otherwise, just return it as is    
        } else {
            return url;
        }
//csc_101701.1_end
    }

    /**
     * Encode a redirect URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param vc the ViewContext
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeRedirectURL(ViewContext vc, String url) {
        if (vc!=null) {
//csc_101701.1_start
/*        
            EventContext ec = vc.getEventContext();
            if (ec!=null && ec instanceof ViewEventContext) {
                ViewEventContext vec = (ViewEventContext) ec;
                return encodeRedirectURL(vec.getRequest(), vec.getResponse(), url);
            }
*/            
            return encodeRedirectURL(vc.getRequest(), vc.getResponse(), url);
//csc_101701.1_end
        }
        return url;
    }
    
    /**
     * Encode a redirect URL if the REWRITE_URLS option is set. Using this option
     * makes it easy to reconfigure Barracuda if your server has problems
     * with URL rewriting (ie. Enhydra 3.x)
     *
     * @param req the servlet request
     * @param resp the servlet response
     * @param url the target url
     * @return an encoded url (unless REWRITE_URLS = false, in which case
     *        it will just return the original url value)
     */
    public static String encodeRedirectURL(HttpServletRequest req, HttpServletResponse resp, String url) {
//csc_101701.1_start - This does not seem to work...the 
//isRequestedSessionIdValid() is always returning false in Tomcat 3.3...
//        if (REWRITE_URLS &&
//            req.isRequestedSessionIdValid() &&
//            !req.isRequestedSessionIdFromCookie()) return resp.encodeRedirectURL(url);
//        else return url;

        //This check basically says that if we want to rewrite URLs AND we have a
        //session AND the session id didn't come from a cookie, then go ahead and 
        //encode the url
        HttpSession session = req.getSession(false);
        if (REWRITE_URLS &&
            session!=null &&
            !req.isRequestedSessionIdFromCookie()) {
            return resp.encodeRedirectURL(url);
            
        //otherwise, just return it as is    
        } else {
            return url;
        }
//csc_101701.1_end
    }
}    
