package org.enhydra.barracuda.examples.xmlc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.enhydra.xml.xmlc.*;

import org.enhydra.barracuda.core.comp.*;
import org.enhydra.barracuda.core.comp.helper.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.enhydra.barracuda.core.event.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.data.*;

/**
 * Here's an example of how the component stuff can be implemented
 * in a standalone servlet.
 */
public class CompEx4 extends ComponentGateway {

    private ListenerFactory getLinkFactory =     new DefaultListenerFactory() {public BaseEventListener getInstance() {return new GetLinkHandler();}            public String getListenerID() {return getID(GetLinkHandler.class);}};

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(CompEx4HTML.class);

        //create the component and bind it to the views
        BTemplate templateComp = new BTemplate(new SimpleModel());
        templateComp.setView(new DefaultTemplateView(page.getElementById("TemplateID")));
        root.addChild(templateComp);

        //return the page
        return page;
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(true);
    }

    //--------------- UserReportModel ------------------------
    /**
     * SimpleModel
     */
    class SimpleModel extends AbstractTemplateModel {
        //register the model by name
        public String getName() {
            return "SimpleModel";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Link1")) {
                BAction baction = new BAction();
                baction.addView(new DefaultView(vc.getTemplateNode().cloneNode(true)));
                baction.addEventListener(getLinkFactory);
                baction.setParam("foo1","blah1");
                baction.setParam("foo2","blah2");
                return baction;
            } else if (key.equals("Link2")) {
//                BAction baction = new BAction("http://blah.com/testLink2.html", new DefaultView(vc.getTemplateNode().cloneNode(true)));
                BAction baction = new BAction("http://blah.com/testLink2.html");
                return baction;
            } else if (key.equals("Link3")) {
//                BLink blink = new BLink("link example (event)", vc);
                BLink blink = new BLink("link example (event)");
                blink.addEventListener(getLinkFactory);
                blink.setParam("foo1","blah1");
                blink.setParam("foo2","blah2");
                return blink;
            } else if (key.equals("Link4")) {
//                BLink blink = new BLink("link example (url)", "http://blah.com/testBLink.html", vc);
                BLink blink = new BLink("link example (url)", "http://blah.com/testBLink.html");
                return blink;
            } else if (key.equals("Link5")) {
//                BAction baction = new BAction("http://blah.com/testLink2.html", new DefaultView(vc.getTemplateNode().cloneNode(true)));
                BAction baction = new BAction("http://blah.com/testLink2.html");
                baction.setEnabled(false);
                return baction;
            } else if (key.equals("Form1")) {
//                BAction baction = new BAction("http://blah.com/formTest1.html", new DefaultView(vc.getTemplateNode().cloneNode(true)));
                BAction baction = new BAction("http://blah.com/formTest1.html");
                return baction;
            } else if (key.equals("Button1")) {
//                BAction baction = new BAction(new DefaultView(vc.getTemplateNode().cloneNode(true)));
                BAction baction = new BAction();
                baction.addEventListener(getLinkFactory);
                return baction;
            } else if (key.equals("Button2")) {
//                BLink blink = new BLink("link example on a button", "http://blah.com/testBLink.html", vc);
                BLink blink = new BLink("link example on a button", "http://blah.com/testBLink.html");
                return blink;
            } else return super.getItem(key);
        }
    }

    /**
     * GetLinkHandler - handle the request to render the test 
     * screen
     */
    class GetLinkHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {

            //something would actually happen here
        }
    }


}
