/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site (http://www.enhydra.org/).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: LocalizeXmlcUtilsImpl.java,v 1.4 2002/02/06 19:43:28 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

// import project specifics

// import logging specifics
//import org.apache.log4j.Category;

// import java specifics
import java.io.File;
import java.util.StringTokenizer;
import java.util.Arrays;

/**
 * Implementation of the XmlcUtils class for use when localizing files
 *
 * @author Robert Leftwich
 * @version 1.0
 */
public class LocalizeXmlcUtilsImpl extends DefaultXmlcUtilsImpl {

  //===========================================================================
  // CONSTRUCTORS
  //===========================================================================

  /**
   * Construct default LocalizeXmlcUtilsImpl (protected to force the use
   * of XmlcUtils.create() factory method).
   */
  protected LocalizeXmlcUtilsImpl() {
    super();
  }

  //===========================================================================
  // QUERY METHODS
  //===========================================================================

  /**
   * Build a Java class name out of the specified components.
   * This implementation checks to see if the file name is
   * localized (i.e. it has a _xx suffix - where xx is a country
   * code). If it does, it adds the modifier prior to the _xx
   * suffix but after the file name.
   * e.g. config_en becomes configHTML_en if the modifier is HTML
   *
   * @param theFullBaseFileName
   *               The full base file name (specified with '/' separators)
   *               and no file type. e.g. "org.enhydra.test"
   * @param theModifier
   *               The file modifier to append to the class name
   *               e.g. HTML
   * @return A Java class name string
   */
  public String buildClassName(String theFullBaseFileName,
                                String theModifier) {
    // Synthesise class name based on path and filename based on filename
    String baseFileName = theFullBaseFileName;
    String localeStr = "";

    //first and foremost, figure out if we're looking at an
    //existing localized template file. If so, split it into
    //baseFileName and localeStr segments.
    if (theFullBaseFileName.indexOf("_")>0) {
      int offset = 0;
      String tfile = theFullBaseFileName;
      StringTokenizer st = new StringTokenizer(tfile, "_");
      while (st.hasMoreTokens()) {
        String token = st.nextToken();
        int pos = Arrays.binarySearch(Localize.countryCodes, token);
        if (pos>-1 && pos<Localize.countryCodes.length && Localize.countryCodes[pos].equals(token)) break;
        else offset += token.length()+1;
      }
      if (offset<baseFileName.length()) {
        baseFileName = theFullBaseFileName.substring(0,offset-1);
        localeStr = theFullBaseFileName.substring(offset-1);
      }
      //theirLogger.debug("base filename:"+baseFileName+" localeStr:"+localeStr+" offset:"+offset);
    }

    String className = baseFileName.replace('/', '.');
    className = className.replace(File.separatorChar, '.') + theModifier + localeStr;
    return className;
  }

  //===========================================================================
  // MUTATOR METHODS
  //===========================================================================

  //===========================================================================
  // HELPER METHODS
  //===========================================================================

  //===========================================================================
  // DATA MEMBERS
  //===========================================================================

  //===========================================================================
  // STATIC DATA MEMBERS
  //===========================================================================

  /**
   * The loggger for this class
   */
//  static Category theirLogger = Category.getInstance(LocalizeXmlcUtilsImpl.class.getName());

}
