/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestDateValidator.java,v 1.7 2002/06/04 14:31:17 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.forms.validators;

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.forms.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.testbed.*;

/**
 * This test verifies that DateValidator works correctly.
 */
public class TestDateValidator extends ValidatorTestCase {

	//common vars (customize for every test class)
	private static String testClass = TestDateValidator.class.getName();
	private static Logger logger = Logger.getLogger("test."+testClass);

	//variables
	
	//-------------------- Basics --------------------------------

	/**
	 * Public Constructor
	 */
	public TestDateValidator(String name) {
		super(name);
	}
	
	/**
	 * Every test class should have a main method so it can be run
	 * directly (when debugging tests you often will not want to run
	 * the whole suite)
	 *
	 * @param args defined in test.util.TestUtil
	 */
	public static void main(String args[]) {
		//check for standard runtime parameters
		TestUtil.parseParams(args);

		//launch the test
		if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
		else junit.swingui.TestRunner.main(new String[] {testClass});
	}

	public static Test suite() {

		TestSuite allLocalesSuite = new TestSuite();

		Locale[] availableLocales = Locale.getAvailableLocales();
		for (int i = 0; i < availableLocales.length; i++) {
			TestSuite suite = new TestSuite();
			suite.addTestSuite( TestDateValidator.class );
			allLocalesSuite.addTest( new LocaleTestSuiteSetup( suite, availableLocales[i]) );
			logger.info("Adding:" + availableLocales[i].getCountry() );
		}

		return allLocalesSuite;
	}
	
	//-------------------- Actual Tests --------------------------
	//Note: all the methods herein should follow the testXXXX naming convention
	//Also keep in mind that local vars set in one test method are NOT retained
	//when the next method is invoked because JUnit makes a separate instance of
	//the test class for each testXXXX method!!!
	
	/**
	 * Test String types
	 */
	public void testString() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.STRING);

    // ilc_022502.1_start
    // added going through the mapping process
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("InvalidString1", "foo");
    sm.putState("InvalidString2", TestUtil.dateStringInDefaultLocaleShortForm("2000", "1", "32"));
    sm.putState("InvalidString3", TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "1"));
    sm.putState("InvalidString4", TestUtil.dateStringInDefaultLocaleShortForm("2001", "2", "29"));
    sm.putState("InvalidString5", TestUtil.dateStringInDefaultLocaleShortForm("2000", "01", "32"));
    sm.putState("InvalidString6", TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "01"));
    sm.putState("InvalidString7", TestUtil.dateStringInDefaultLocaleShortForm("2001", "02", "29"));
    sm.putState("InvalidString8", TestUtil.dateStringInDefaultLocaleShortForm("2002", "01", "32"));
    sm.putState("InvalidString9", TestUtil.dateStringInDefaultLocaleShortForm("2010", "13", "01"));
    sm.putState("InvalidString10", TestUtil.dateStringInDefaultLocaleShortForm("2013", "02", "29"));


    this.assertAllInvalid(v, sm, FormType.STRING);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("ValidString1", "");
    sm.putState("ValidString2", null);
    sm.putState("ValidString3", TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"));
    sm.putState("ValidString4", TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1"));
    sm.putState("ValidString5", TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "1"));
    sm.putState("ValidString6", TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"));
    sm.putState("ValidString7", TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"));
    sm.putState("ValidString8", TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"));
    sm.putState("ValidString9", TestUtil.dateStringInDefaultLocaleShortForm("2010", "12", "31"));
    sm.putState("ValidString10", TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01"));
    sm.putState("ValidString11", TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "01"));
    sm.putState("ValidString12", TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "01"));
    sm.putState("ValidString13", TestUtil.dateStringInDefaultLocaleShortForm("2000", "02", "29"));
    sm.putState("ValidString14", TestUtil.dateStringInDefaultLocaleShortForm("2004", "02", "29"));

    this.assertAllValid(v, sm, FormType.STRING);

    // ilc_022502.2_start
    // replaced with mapping values to test
    /*
		assertAllInvalid(v, el,
										 new String[] {
                                    //ilc_022102.1_start
                                    //the way we define null "  " should pass
                                    //"   ",
                                    //ilc_022102.2_end
																		"foo",
																		TestUtil.dateStringInDefaultLocaleShortForm("2000", "1", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "1"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2001", "2", "29"),

																		TestUtil.dateStringInDefaultLocaleShortForm("2000", "01", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "01"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2001", "02", "29"),

																		TestUtil.dateStringInDefaultLocaleShortForm("2002", "01", "32"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2010", "13", "01"),
																		TestUtil.dateStringInDefaultLocaleShortForm("2013", "02", "29"),
																	} );


		assertAllValid(v, el,
									 new String[] { "",
										              null,
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "1"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2010", "12", "31"),

																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2001", "12", "01"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2000", "02", "29"),
																	TestUtil.dateStringInDefaultLocaleShortForm("2004", "02", "29"),
																} );
    */
    // ilc_022502.2_end
	}

	public void testLong() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.LONG);

    // ilc_022502.3_start
    // do testing by going through the mapping process
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("InvalidLong1", new Long(0));
    sm.putState("InvalidLong1", new Long(1));
    sm.putState("InvalidLong1", new Long(-1));
    sm.putState("InvalidLong1", new Long(Long.MIN_VALUE));
    sm.putState("InvalidLong1", new Long(Long.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.LONG);
    // ilc_022502.3_start

    // ilc_022502.4_start
    // replaced with mapping values to test
    /*
		assertAllInvalid(v, el, new Long[]{ new Long(0),
																				new Long(1),
																				new Long(-1),
																				new Long(Long.MIN_VALUE),
																				new Long(Long.MAX_VALUE),
																			} );

		assertAllValid(v, el, new Long[]{ null, } );
    */
    // ilc_022502.4_end
	}
	
	public void testBoolean() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.BOOLEAN);

    // ilc_022502.5_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("BooleanInvalid1", Boolean.FALSE);
    sm.putState("BooleanInvalid2", Boolean.TRUE);

    this.assertAllInvalid(v, sm, FormType.BOOLEAN);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("BooleanValid1", null);
    this.assertAllValid(v, sm, FormType.BOOLEAN);

    // ilc_022502.5_end

    // ilc_022502.6_start
    /* commented out to use statemaps instead
		assertAllInvalid(v, el, new Boolean[] { Boolean.FALSE,
																						Boolean.FALSE,
																					} );

		assertAllValid(v, el, new Boolean[]{ null, } );
    */
    // ilc_022502.6_end

	}
	
	public void testShort() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.SHORT);
    // ilc_022502.7_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("ShortInvalid1", new Short((short)0));
    sm.putState("ShortInvalid2", new Short((short)1));
    sm.putState("ShortInvalid3", new Short((short)-1));
    sm.putState("ShortInvalid4", new Short(Short.MIN_VALUE));
    sm.putState("ShortInvalid5", new Short(Short.MAX_VALUE));


    this.assertAllInvalid(v, sm, FormType.SHORT);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("ShortValid1", null);
    this.assertAllValid(v, sm, FormType.SHORT);

    // ilc_022502.7_end

    // ilc_022502.8_start
    /* commented out to use statemaps instead

		assertAllInvalid(v, el, new Short[] { new Short((short)0),
																					new Short((short)1),
																					new Short((short)-1),
																					new Short(Short.MIN_VALUE),
																					new Short(Short.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Short[]{ null, } );
    */
    // ilc_022502.8_end
	}
	
	public void testInteger() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.INTEGER);
    // ilc_022502.9_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("IntegerInvalid1", new Integer(0));
    sm.putState("IntegerInvalid2", new Integer(1));
    sm.putState("IntegerInvalid3", new Integer(-1));
    sm.putState("IntegerInvalid4", new Integer(Integer.MIN_VALUE));
    sm.putState("IntegerInvalid5", new Integer(Integer.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.INTEGER);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("IntegerValid1", null);
    this.assertAllValid(v, sm, FormType.INTEGER);

    // ilc_022502.9_end

    // ilc_022502.10_start
    /* commented out to use statemaps instead
		assertAllInvalid(v, el, new Integer[] { new Integer(0),
																						new Integer(1),
																						new Integer(-1),
																						new Integer(Integer.MIN_VALUE),
																						new Integer(Integer.MAX_VALUE),
																					} );

		assertAllValid(v, el, new Integer[]{ null, } );
    */
    // ilc_022502.10_end
	}
	
	public void testDate()  {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.DATE);

		DateFormat aDateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		aDateFormat.setLenient(false);


    // ilc_022202.11_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    try {
      sm.putState("DateValid1", null);
      sm.putState("DateValid1", new Date());
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01")));
      sm.putState("DateValid1", aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "1")));

    } catch(java.text.ParseException ex) {
      fail( "testDate failed because " + ex.getMessage() );
    }
    // ilc_022502.11_end

    // ilc_022502.12_start
    /* commented out to use statemaps instead
		try {
			// NOTE - no invalid date test as it should not be possible to create an invalid date
      assertAllInvalid(v, el,
											 new Date[] {
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "1", "32")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "1")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "2", "29")),

													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "01", "32")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "13", "01")),
													aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "02", "29")),
                        } );

			assertAllValid(v, el,
										 new Date[]{
												null,
												new Date(),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "1", "1")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2000", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2004", "2", "29")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("1999", "12", "31")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2001", "01", "01")),
												aDateFormat.parse(TestUtil.dateStringInDefaultLocaleShortForm("2005", "01", "1")),
															 } );
		} catch ( java.text.ParseException ex )  {
			fail( "testDate failed because " + ex.getMessage() );
		}
    */
    //// ilc_022502.12_end
	}
	
	public void testDouble() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.DOUBLE);
    // ilc_022502.13_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("DoubleInvalid1", new Double(0));
    sm.putState("DoubleInvalid2", new Double(1));
    sm.putState("DoubleInvalid3", new Double(-1));
    sm.putState("DoubleInvalid4", new Double(Double.MIN_VALUE));
    sm.putState("DoubleInvalid5", new Double(Double.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.DOUBLE);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("DoubleValid1", null);
    this.assertAllValid(v, sm, FormType.DOUBLE);

    // ilc_022502.13_end

    // ilc_022502.14_start
    /* commented out to use statemaps instead
		assertAllInvalid(v, el, new Double[] {  new Double(0),
																						new Double(1),
																						new Double(-1),
																						new Double(Double.MIN_VALUE),
																						new Double(Double.MAX_VALUE),
																				 } );

		assertAllValid(v, el, new Double[]{ null, } );
    */
    // ilc_022502.14_end
	}
	
	public void testFloat() {
		DateValidator v = new DateValidator();
		DefaultFormElement el = new DefaultFormElement("key", FormType.FLOAT);
    // ilc_022502.15_start
    // use a StateMap for testing to make sure elements go through mapping
    DefaultStateMap sm = new DefaultStateMap();

    sm.putState("FloatInvalid1", new Float(0));
    sm.putState("FloatInvalid2", new Float(1));
    sm.putState("FloatInvalid3", new Float(-1));
    sm.putState("FloatInvalid4", new Float(Float.MIN_VALUE));
    sm.putState("FloatInvalid5", new Float(Float.MAX_VALUE));

    this.assertAllInvalid(v, sm, FormType.FLOAT);

    sm = null;
    sm = new DefaultStateMap();
    sm.putState("FloatValid1", null);
    this.assertAllValid(v, sm, FormType.FLOAT);

    // ilc_022502.15_end

    // ilc_022502.16_start
    /* commented out to use statemaps instead
		assertAllInvalid(v, el, new Float[] { new Float(0),
																					new Float(1),
																					new Float(-1),
																					new Float(Float.MIN_VALUE),
																					new Float(Float.MAX_VALUE),
																				} );

		assertAllValid(v, el, new Float[]{ null, } );
    */
    // ilc_022502.16_end
	}

}
