/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: PDataTestCases.java,v 1.6 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;

import junit.framework.*;

import org.apache.log4j.*;
import org.enhydra.barracuda.testbed.*;


/**
 * Test case for any PData. The idea is that you can test any
 * PData object by extending this test case and adding tests 
 * for the specific implementation details. All you really have 
 * to do is override teh factory PData methods.
 */
public abstract class PDataTestCases extends StateMapTestCases {
    //common vars (customize for every test class)
//    protected static String testClass = PDataTestCases.class.getName();
//    protected static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public PDataTestCases(String name) {
        super(name);
    }
    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Verify the parental aspects
     */
    public void testParental() {
        if (logger.isInfoEnabled()) logger.info("testing parental methods");
        PData proot = null;
        PData pchild1 = null;
        PData pchild2 = null;
        PData pchild3 = null;

        //test simple set/get
        proot = getPDataInstance();
        pchild1 = getPDataInstance();
        pchild1.setParent(proot);
        assertTrue("Parental check 1 failed...basic set/get failure", pchild1.getParent()==proot);
        pchild1.setParent(null);
        assertTrue("Parental check 1a failed...set to null failure", pchild1.getParent()==null);
        pchild1.setParent(pchild1);
        assertTrue("Parental check 1b failed...allowed parent to be set to self", pchild1.getParent()!=pchild1);

        //test the inheritParents attribute set/get
        proot = getPDataInstance();
        assertTrue("Parental check 2 failed...invalid default inheritParents value", proot.isInheritParents()==true);
        proot.setInheritParents(false);
        assertTrue("Parental check 2a failed...basic set/get failure", proot.isInheritParents()==false);
        proot.setInheritParents(true);
        assertTrue("Parental check  2b failed...basic set/get failure", proot.isInheritParents()==true);
        
        //test the getRootParent
        proot = getPDataInstance();
        pchild1 = getPDataInstance();
        pchild1.setParent(proot);
        pchild2 = getPDataInstance();
        pchild2.setParent(pchild1);
        assertTrue("Parental check 3 failed...root parent not correctly located", pchild2.getRootParent()==proot);
    }
    
    /**
     * Verify the PData is serializable
     */
    public void testSerializable() {
        if (logger.isInfoEnabled()) logger.info("testing for Serializable");
        PData pdata = this.getPDataInstance();
        assertTrue("PData instance does not implement Serializable!", pdata instanceof Serializable);
    }
    

    //-------------------- Abstract methods ----------------------
    public abstract PData getPDataInstance();
    public abstract PList getPListInstance();
    public abstract PMap getPMapInstance();

}
