/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestObjectRepositoryAssembler.java,v 1.1 2002/05/31 19:55:59 cryd0221 Exp $
 */
package org.enhydra.barracuda.core.util.data;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.Collections;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import org.enhydra.barracuda.testbed.*;


/**
 * This test verifies the Param object
 */
public class TestObjectRepositoryAssembler extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestObjectRepositoryAssembler.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestObjectRepositoryAssembler(String name) {
        super(name);
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Simple test to make sure the Param object works
     */
    public void testAssembler() {
        if (logger.isInfoEnabled()) logger.info("testing assembler");

        //get the initial values
        String INIT_TEST_STRING = ObjectRepositoryAssembler.TEST_STRING;
        int INIT_TEST_INT = ObjectRepositoryAssembler.TEST_INT;
        boolean INIT_TEST_BOOLEAN = ObjectRepositoryAssembler.TEST_BOOLEAN;

        //assemble into the default ObjectRepository
        new ObjectRepositoryAssembler().assemble("../WEB-INF/object-repository.xml");

        //now make sure the values got changed
        assertTrue("Failed to set TEST_STRING", !ObjectRepositoryAssembler.TEST_STRING.equals(INIT_TEST_STRING));
        assertTrue("Failed to set TEST_INT", ObjectRepositoryAssembler.TEST_INT!=INIT_TEST_INT);
        assertTrue("Failed to set TEST_BOOLEAN", ObjectRepositoryAssembler.TEST_BOOLEAN!=INIT_TEST_BOOLEAN);

        //todo: we really ought to be testing the other aspects of this class as well (instantiating
        //an object, invoking methods on it, and placing it in the object repository)


    }
}
