/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestScriptingType.java,v 1.5 2002/05/29 06:17:36 JacobK Exp $
 */
package org.enhydra.barracuda.core.view;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.servlet.*;
import javax.servlet.http.*;

import org.w3c.dom.*;
import junit.framework.*;

import org.enhydra.barracuda.core.util.*;
import org.enhydra.barracuda.core.util.data.*;
import org.enhydra.barracuda.core.util.dom.*;
import org.apache.log4j.*;
import org.enhydra.barracuda.core.view.*;
import org.enhydra.barracuda.examples.xmlc.*;
import org.enhydra.barracuda.testbed.*;
import org.enhydra.barracuda.testbed.servlet.*;


/**
 * This test verifies that the hierachical organization of 
 * ScriptingType classes. We do this by testing for what it should be 
 * an instanceof and what it shouldn't. The idea is that the ScriptingType
 * extension hierarchy is very important and we don't want it getting 
 * changed accidentally. This test should preclude that.
 */
public class TestScriptingType extends TestCase {
    //common vars (customize for every test class)
    private static String testClass = TestScriptingType.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    ScriptingType st = null;
    Class targetCl = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestScriptingType(String name) {
        super(name);
    }

    public TestScriptingType(String name, ScriptingType ist, Class itargetCl) {
        super(name);
        st = ist;
        targetCl = itargetCl;
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //add the tests
        //...JAVASCRIPT_1x
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.None.class));
        //...JAVASCRIPT_1_0
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.None.class));
        //...JAVASCRIPT_1_1
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.None.class));
        //...JAVASCRIPT_1_2
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.None.class));
        //...JAVASCRIPT_1_3
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript13.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.None.class));
        //...WMLSCRIPT_1x
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.None.class));
        //...WMLSCRIPT_1_0
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.None.class));
        //...WMLSCRIPT_1_1
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.None.class));
        //...WMLSCRIPT_1_2
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.None.class));
        //...NONE
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.NONE, ScriptingType.None.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.NONE, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.NONE, ScriptingType.WmlScript.class));

        //finally return the suite
        return suite;
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Make sure that the class being tested IS an instance of the 
     * target class
     */
    public void testInstance() {
        if (logger.isInfoEnabled()) logger.info("testing instanceof");
        assertTrue(st+" not an instanceof "+targetCl, targetCl.isAssignableFrom(st.getClass()));
    }

    /**
     * Make sure that the class being tested is NOT an instance of the 
     * target class
     */
    public void testNotInstance() {
        if (logger.isInfoEnabled()) logger.info("testing !instanceof");
        assertTrue(st+" is an instanceof "+targetCl+" (and it shouldn't be!)", !targetCl.isAssignableFrom(st.getClass()));
    }
}
