/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id: TestLocalizeFileNames.java,v 1.5 2001/09/25 19:05:31 cryd0221 Exp $
 */
package org.enhydra.barracuda.taskdefs;

// import junit specifics
import junit.framework.TestCase;

// import logging specifics
import org.apache.log4j.Category;

// import java specifics
import java.io.File;
import java.io.IOException;

/**
 * Unit tests for buildClassName() method of LocalizeXmlcUtilsImpl
 */
public class TestLocalizeFileNames extends TestCase {

  //===========================================================================
  // CONSTRUCTORS
  //===========================================================================

    /**
     * Default constructor
     *
     * @param theName   The name of the test
     */
    public TestLocalizeFileNames( String theName) {
        super( theName );
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     */
    protected void setUp() throws Exception {
        super.setUp();
        // save default xmlc utils class
        this.myOriginalDefautlUtilsClass =
          System.getProperty( XmlcUtils.DEFAULT_UTILS_CLASS_KEY, "" );
        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY,
                       LocalizeXmlcUtilsImpl.class.getName());
    }

    /**
     * Tears down the fixture, for example, close a network connection.
     * This method is called after a test is executed.
     */
    protected void tearDown() throws Exception {
        super.tearDown();

        // restore default xmlc utils class
        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY,
                       this.myOriginalDefautlUtilsClass);
    }

    //===========================================================================
    // TEST METHODS
    //===========================================================================
    public void testBuildClassName() throws IOException {
        XmlcUtils xmlcUtils = XmlcUtils.create();
        assertEquals( "Should have localize implementation",
                      LocalizeXmlcUtilsImpl.class,
                      xmlcUtils.getClass() );

        String filename = "config";
        String typeModifier = "HTML";
        String expectedClassName = "configHTML";
        String newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 0 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_aa";
        expectedClassName = "configHTML_aa";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 1 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_test";
        expectedClassName = "config_testHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 2 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_eng";
        expectedClassName = "config_engHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 3 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "config_aaab";
        expectedClassName = "config_aaabHTML";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 4 should be as expected",
                     expectedClassName,
                     newClassName );

        filename = "name1/name2/name1/name2/config_zu";
        expectedClassName = "name1.name2.name1.name2.configHTML_zu";
        newClassName = xmlcUtils.buildClassName( filename, typeModifier );
        assertEquals("Class 5 should be as expected",
                     expectedClassName,
                     newClassName );
    }


    //===========================================================================
    // DATA MEMBERS
    //===========================================================================
    private String myOriginalDefautlUtilsClass;

    //===========================================================================
    // STATIC DATA MEMBERS
    //===========================================================================
    // setup the loggger for this class
    static Category theirLogger = Category.getInstance( TestLocalizeFileNames.class.getName() );

}
